function [success,targetLocations,numberOfScans,goalInformation] = ContinueRunMission(showFigures, pathToLoad,pathToSave)

% Runs a single mission of Type2
% Return
% success: 0/1 (no targets found/one or more targets found)
% targetLocations: where targets were found; initialized only if success=1
% numberOfScans: for every searcher records the number of scans it
% performed
% goalInformation: sequence of goal locations the searcher(s) chose during
% the mission



global TicksAllowed;
global robotBounds;
global numRobots;
global scanCost;
global tree;
global target;
%global belief_evol;
global FLAG_SHOW_MAP;
global FLAG_SHOW_3D;
global gcmesh;
global threeDHandler;
%global pnull
global maxDepth;
%global GoalHistory
global scanResult;
global stop_epsilon;

%loads all variables inside the mat file specified by path
%the regular expression is used to avoid loading some variables
load(pathToLoad,'-regexp','^(?!showFigures|stop_epsilon|pathToSave|pathToLoad).');

if showFigures
    FLAG_SHOW_MAP = true;
    FLAG_SHOW_3D = true;
else
    FLAG_SHOW_MAP = false;
    FLAG_SHOW_3D = false;
end

%stuff for figures if they are used
if(FLAG_SHOW_MAP)
    gcmesh = figure;
    PlotEstimate(gcmesh);

end

if(showFigures)
    if(FLAG_SHOW_3D)
        [threeDHandler h] = Initialize3DPlot(numRobots);
    end
end


isSearchDone = 0;

success = 0;


while ~isSearchDone
 %   N = N + 1
    
    if(makeMovie)
        %advance the clock by 1 time step so robot motion can be displayed
        value = min(min(TicksTillGoal),1);
        TicksTillGoal(:) = TicksTillGoal(:) - value;
        TicksAllowed(:) = TicksAllowed(:) - value;
    else
        %advance the clock by the smallest time left
        value = min(TicksTillGoal);
        TicksTillGoal(:) = TicksTillGoal(:) - value;
        TicksAllowed(:) = TicksAllowed(:) - value;
    end
    
    %are there any tics left
    if sum(TicksAllowed <= 0) > 0
        disp('Out of time');
        break;
    end;
    %Every robot that is at its goal gets to switch states
    for i=1:numRobots
        
        %update the robot locations
        if(TotalTicksTillGoal(i) == 0)
            perc = 1;
        else
            perc = (TotalTicksTillGoal(i) - TicksTillGoal(i))/TotalTicksTillGoal(i);
        end
        robotPositions(1,i) = tree(Start_Pos(i)).xc + ...
            (tree(Goal_Pos(i)).xc-tree(Start_Pos(i)).xc)*perc;
        robotPositions(2,i) = tree(Start_Pos(i)).yc + ...
            (tree(Goal_Pos(i)).yc-tree(Start_Pos(i)).yc)*perc;
        robotPositions(3,i) = tree(Start_Pos(i)).depth + ...
            (tree(Goal_Pos(i)).depth-tree(Start_Pos(i)).depth)*perc;
        
        if TicksTillGoal(i) <= 0
            if(Robot_State(i) == 0) %need a new goal
                Start_Pos(i) = Goal_Pos(i);
                Goal_Pos(i) = GetMaximumPosition(Start_Pos(i),robotBounds(i,:));
                GoalHistory = [ GoalHistory Goal_Pos(i)];
                TicksTillGoal(i) = ceil(StraightLineDistance(Start_Pos(i),Goal_Pos(i)));
                TotalTicksTillGoal(i) = TicksTillGoal(i);
                Robot_State(i) = 1;
                %log the robots new goal
                goalInformation{i} = [goalInformation{i} [tree(Goal_Pos(i)).depth;tree(Goal_Pos(i)).xc;tree(Goal_Pos(i)).yc]];
            elseif (Robot_State(i) == 1) %start scanning
                Robot_State(i) = 2;
 
                [z,pos] = GetSensorReading(tree(Goal_Pos(i)).xc, tree(Goal_Pos(i)).yc, target);
                scanResult(i,1) = z;
                scanResult(i,2) = pos;    
                
                TicksTillGoal(i) = scanCost;
                TotalTicksTillGoal(i) = 0; %set to 0 so the robots dont twitch on the gui
            else %finish a scan
                numberOfScans(i) = numberOfScans(i) + 1;
                %fprintf('R %d: Scan number %d\n',i,numberOfScans(i));
%                 z = scanResult(i,1);
%                 pos = scanResult(i,2);
                UpdatePosterior(z,pos);
                [isSearchDone, cleared_cells, targetLocations] = ...
                    EvaluateSearchDecision( z, Goal_Pos(i), cleared_cells );
                if (isSearchDone)
                    disp('Target Located');
                    success = 1;
                    break;
                end;
                Robot_State(i) = 0;
                TicksTillGoal(i)= 0;
            end
            
        end;
    end;
    %  belief_evol = [belief_evol; tree(1).p];
    
    %make sure the tree is still consistent
    CheckTreeConsistency
    if ~showFigures
        continue;
    end
    
    
    %=========================
    % Visualization handling
    %=========================
    if ( FLAG_SHOW_MAP )
        % if mod(i,10) == 0 % plot once in a while to save time...
        PlotEstimate(gcmesh);
        
        %  end
        
        % Annotating desired search location on probability map
        %  - need to recreate object since probability map figure is redrawn
        figure(gcmesh); hold on;
        for i=1:numRobots
            rString = 'rx';
            if Robot_State(i) ~= 0
                %show a blue x on no detection and a green x on detection
                if(scanResult(i,1)==0)
                    rString = 'bx';
                else
                    rString = 'gx';
                end
            end
            
            robotX = robotPositions(1,i);
            robotY = robotPositions(2,i);
            h_goal = plot3(robotX, robotY, max(get(gca, 'ZLim')), rString, 'MarkerSize', 10 );
            plot3([tree(Start_Pos(i)).xc;robotX], [tree(Start_Pos(i)).yc;robotY], [max(get(gca, 'ZLim'));max(get(gca, 'ZLim'))]);
        end
        %saveas(gcmesh,strcat('/tmp/ramdisk/',num2str(N)),'jpg');
        if makeMovie
            Movie(N+1) = getframe(gcmesh);
        end
        hold off;
    end         % end if(check FLAG_SHOW_MAP flag)
    
    if(FLAG_SHOW_3D)
        UpdatePos3D(robotPositions,threeDHandler,h,Robot_State,scanResult);
        if makeMovie
            Movie2(N+1) = getframe(threeDHandler);
        end
    end
end

if makeMovie
    disp('writing the prior movie');
    movie2avi(Movie(1:(N+1)),'prior');
    
    disp('writing the 3d movie');
    movie2avi(Movie2(1:(N+1)),'3d');
end
fprintf('Saving results to %s\n',pathToSave);
save(pathToSave);

end



