

RUNS = 100;

% depth-dependent sensor performance
global alpha; % false positive
global beta;  % missed detection
% sensor perfomance
x = 1:10;
% alpha values parametrized by depth
alpha = 0.2 - 0.01*x;
% beta values parametrized by depth
beta = 0.3 - 0.02*x;

global lower_thresh;
global upper_thresh;
% Lower search decision thresholds for clearing a cell
lower_thresh = 0.0 + 0.005*x;
% Upper search decision thresholds for designating a cell as occupied
upper_thresh = 0.81 + 0.02*x;

global StopCriterion;
% Stop criterion codes
% 1: stop when all cells have an entropy below a certain threshold, i.e.
% H(cell_confidence_requirement)
% 2: stop when the normalized tree entropy is below entropyFraction.
% Normalized tree entropy is the sum of entropy on the leaves divided by
% the number of leaves * H(globalEntropyLevel)
StopCriterion = 3;


% Costs for misclassificaions and correct classifications
global C00;
global C11;
global C10; % cost for false positive
global C01; % cost for missed detection
C11 = 0;
C00 = 0;
C10 = 10;
C01 = 1000;


global search_strategy;
% Search strategy codes
% 1: ICRA heuristic
% 2: Pure information gain
% 3: information gain with distance
search_strategy = 3;

% threshold to establish when two reals are the same
global epsilon;
epsilon = 0.00001;

global TicksAllowed; % search time left for each searcher
% overall search budget (aka maximum mission length)
maxTicks = 25000;

global numRobots; % number of searchers
numRobots = 1;
global robotBounds; % bounds for each searcher
robotBounds = [0 0 256 256];


pathToLoad = '100RunsType2StoppingCriterion3Factor001';
pathToSave = '100RunsType2StoppingCriterion3Factor0005';


% load precomputed target positions; see generateTargetPositions.m
VariableTargets = load('TargetPositions.mat');
VariableTargets = VariableTargets.targets;

success = zeros(1,RUNS);
% stores the number of steps needed to complete the i-th run
time_steps = zeros(1,RUNS);
% stores the target positions found by the search algorithm
target_locations = cell(1,RUNS);
% number of targets found
num_targets_found = zeros(1,RUNS);
% number of false positives
num_false_positives = zeros(1,RUNS);
% number of missed targets
num_missed_targets = zeros(1,RUNS);
% cost of the mission
mission_cost = zeros(1,RUNS);

num_scans = zeros(numRobots,RUNS);
goal_information = cell(1,RUNS);
number_of_scans = cell(1,RUNS);

global stop_epsilon;
stop_epsilon = 0.005;

for newRuns = 81:100
    target = VariableTargets{newRuns};
    path = sprintf('%s/RUN%d',pathToLoad,newRuns);
    savePath = sprintf('%s/RUN%d',pathToSave,newRuns);
    [success(newRuns),target_locations{newRuns},number_of_scans{newRuns},goal_information{newRuns}] = ContinueRunMission(false,path,savePath);
    
    num_targets_found(newRuns) = size(target_locations{newRuns},2);
    time_steps(newRuns) = maxTicks - TicksAllowed(1);
   
    [mission_cost(newRuns) num_missed_targets(newRuns) num_false_positives(newRuns)] = ComputeDecisionCosts(target_locations{newRuns},target);
    fprintf('%d: Ticks taken %f\n',newRuns,time_steps(newRuns));
    fprintf('%d: Incurred Cost %f\n',newRuns,mission_cost(newRuns));
    save '100RunsType2StoppingCriterion3Factor008'
end
