function n = CreateNode(parent,initprob,pos)

%
% Creates a new node and returns it
% parent: index of the parent node
% initprob: probability of being free
% pos: determines where it is located within the parent's region

% This is the main data structure. Every tree node has the following fields
% p: probabilty of being occupied 
% leaf: boolean value (1 leaf/0 internal node)
% NW,NE,SW,SE: four pointers to the children (meaningful only if leaf = 0)
% xmin,ymin,xmax,ymax: boundaries of the region
% parent: pointer to the parent
% xc,yc: coordinates of the center of the region
% depth: depth in the tree
% cleared: boolean value (1 if cleared/0 if can still be searched)
% decision: boolean value indicating ongoing decision process; should be
% considered only for leaves at the maximum depth

global tree;
global maxDepth;

n = struct('p',initprob,'leaf',1,'NW',0,'NE',0,'SW',0,'SE',0,'xmin',0,...
    'ymin',0,'xmax',0,'ymax',0,'parent',parent,'xc',0,'yc',0,'depth',1,...
    'cleared',0,'tempP',0,'positiveScans',0,'negativeScans',0,'decision',0,...
    'lastScan',0);

if parent ~= 0
    p = tree(parent);
    deltax = (p.xmax-p.xmin)/2;
    deltay = (p.ymax-p.ymin)/2;
    n.depth = p.depth+1;
    if (n.depth > maxDepth)
        error('Maximum depth exceeded');
    end
    switch pos
        case 1 % North West
            n.xmin = p.xmin;
            n.xmax = p.xmin + deltax;
            n.ymax = p.ymax;
            n.ymin = p.ymax - deltay;
            
        case 2 % North East
            n.xmax = p.xmax;
            n.xmin = p.xmax - deltax;
            n.ymax = p.ymax;
            n.ymin = p.ymax - deltay;
            
        case 3 % South East
            n.xmax = p.xmax;
            n.xmin = p.xmax - deltax;
            n.ymin = p.ymin;
            n.ymax = p.ymin + deltay;
            
            
        case 4 % South West
            n.xmin = p.xmin;
            n.xmax = p.xmin + deltax;
            n.ymin = p.ymin;
            n.ymax = p.ymin + deltay;
            
    end
end

n.xc = (n.xmin+n.xmax)/2;
n.yc = (n.ymin+n.ymax)/2;
