function [flag, cleared_cells, targetLocations] = EvaluateSearchDecisionUniform(z,c_curr, cleared_cells)

% Determines if search decision criteria are met
%  - positive observation (z=1) --> split node if possible
%  - negative observation (z=0) --> check lower threshold for clearing
% z: current obs ervation
% c_curr        : current node in tree
% cleared_cells :


global tree;
global maxDepth;
global lower_thresh;
global upper_thresh;
global entropy_thresh;
global cell_confidence_requirement;
global StopCriterion;
global globalEntropyLevel;
global entropyFraction;
global stop_epsilon;

targetLocations = [];
%  - if positive observation, need to spawn children if none exist
%  - if negative observation, need to continue observations to clear location
% if ( z == 1 )       % positive observation
%     % Check if this node has any descendents (i.e., is internal)
%     %  - if so, nothing to do but re-optimize search
%     %  - if not, AND is not at maximum depth, spawn new children and initialize uniformly
%     if ( tree(c_curr).leaf )
%         if (tree(c_curr).depth<maxDepth)
%             current = SplitNode(c_curr);
%         end         % end if-else(check for max depth)
%     end         % end if(check for leaf)
% else                % else(check for null detection)
    
    % Check probability value against threshold appropriate for this depth
    %  -
    if ( tree(c_curr).p <= lower_thresh( tree(c_curr).depth ) )
        % Mark this node and all its descendents cleared
        MarkNodeCleared( c_curr );
        
        % DEBUG: add this cell
        cleared_cells = [cleared_cells; c_curr];
        
    end
    
% end                 % end if-else(z==1)

% chek if search shall be stopped
flag = true;

% generate current targets
l = [tree(1:numel(tree)).leaf];
leafs = tree(l>0);
decisions =  [leafs(1:numel(leafs)).decision];
localized = leafs(decisions>0);
targetLocations = [ localized(1:numel(localized)).xc ; localized(1:numel(localized)).yc ];
numberOfLeaves = numel(leafs);
probs = [leafs(1:numel(leafs)).p];
h = myH(probs);

switch StopCriterion
    case 1  % test on individual cells
        invalid = sum(h>H(cell_confidence_requirement));
        if invalid > 0
            flag = false;
        end
        
    case 2 %average tree entropy
        overallEntropy = sum(h);
        % if ( overallEntropy/(H(globalEntropyLevel)*numberOfLeaves) ) > entropyFraction
        if ( overallEntropy/numberOfLeaves)  > H(cell_confidence_requirement)
            flag = false;
        end
    case 3 % criterion borrowed from Husseings ACC 2009 paper
        % stop when every leaf has entropy smaller than epsilon * J(t)
        overallEntropy = sum(h);
        maxEntropy = numberOfLeaves * max(h);
        J = overallEntropy / maxEntropy;
        threshold = stop_epsilon * J;
        invalid = sum(h>threshold);
%        invalid
%        size(targetLocations,2)
        if invalid > 0
            flag = false;
        end
        
    otherwise
        error('Unknown stopping condition');
end

end
