function [c_next] = GetNextSensorLocation( c_curr, c_goal )

% Determines an existing node adjacent to the current location (regardless
% of depth) which provides the greatest reduction in the depth-weighted
% distance to the goal location
%  c_curr   :  index of node where sensor is currently present
%  c_goal   :  index of node with maximal objective value, not necessarily adjacent
%  c_next   :  index of node, adjacent to c_curr, that best reduces 'distance' to goal


% % DEBUG -- set next sensor location to goal location
% c_next = c_goal;



% Check if the goal location is already the location of the searcher
if (c_curr == c_goal)
    c_next = c_goal;
    
    
    
else        %  otherwise, need to compute next location closer to goal
    
    %-----------------
    % Generate list of all adjacent nodes to the current node
    %
    adj_list = FindAdjacentCells( c_curr );
    
    
    
    
    %-----------------
    % Given adjacency list, find node which minimizes depth-weighted
    %   (Euclidean) distance to goal
    %  - at worst, linear search through entire tree structure
    
    minCost = 10^6;         % init arbitrary cost
    c_next = c_curr;        % init min cost node as current node, i.e., stationary sensor
    for which_pos = 1:length(adj_list)
        
        % Since we know that current location cannot be goal cell, ignore self adjacency (to avoid low distance)
        if (c_curr ~= adj_list(which_pos))
            % Compute distance to goal, including weighting due to ascent/descent
            d = Distance( adj_list(which_pos), c_goal );
            
            % Determine if this location is closer to the goal
            if ( d <= minCost )
                
                % Update to new sensor location
                %  - this location is closer to the destination
                minCost = d;
                c_next = adj_list(which_pos);
                
            end         % end if(check for 'closer' location)
        end         % end if(check for non-stationary next position)
    end         % end for(which_pos)
 %   fprintf('GetNextSensorLocation: minCost=%4.4f at ind=%d\n', minCost, c_next);
    
    
    
end         % end function GetNextSensorLocation()