function LoadFullTree(fname)

%
% Load a full tree with one leaf for each grid cell in the original
% regolarly spaced grid
% fname: file name with prior information; for the moment supports
%  only square regions
% Note: will override global variable tree, if any. 
%  This function may take some times for fine grain grids...



global tree;
global maxDepth;
global epsilon;
global treeFigureHandler;

data = load(fname);

prior = data.init_belief_map;
size_grid = data.SEARCH_GRID_L;

step = size(prior,1)/size_grid /2;

if size(prior,1) ~= size(prior,2)
    error('Prior must be square');
end

oldMaxDepth = maxDepth; % will be restored at the end
maxDepth = log(size_grid^2)/log(4)+1; % log_4 of number of cells
% some auxiliary data structures to perform fast pre-allocation

positions = 1:maxDepth;
for i = 1:numel(positions)
    positions(i) = 4^(i-1);
end

offsets = zeros(size(positions));
offsets(1) = 1;
for i = 2:size(offsets,2);
    offsets(i)= sum(positions(1:i-1))+1;
end


% create the root
initProb = sum(sum(prior));
root = CreateNode(0,initProb,0);
root.xmin = 0;
root.ymin = 0;
root.xmax = size(prior,1);
root.ymax = size(prior,2);
root.xc = (root.xmax + root.xmin)/2;
root.yc = (root.ymax + root.ymin)/2;

tree(sum(positions)) = root; % array preallocation
tree(1) = root; % initializes the root

% create a full tree with one leaf per cell
done = 0;
cur_node = 1;
while done ~= 1
%   tree(cur_node).depth
   if  tree(cur_node).depth == maxDepth 
       done = 1;
   else
       SplitNode(cur_node,offsets);
       cur_node = cur_node+1;
   end
end

%DrawQuadTree(treeFigureHandler);

disp('This Version of LoadFullTree builds a fake tree...');

% copy probability a-priori values into the leaves
for i = 1:numel(tree)
    if tree(i).leaf == 1
        row = tree(i).yc+step;
        col = tree(i).xc+step;
      %  tree(i).p = prior(row,col);
      tree(i).p = 5 / (256^2);
    end
end

% restores tree consistency, i.e. recomputes probability for internal nodes
for i = numel(tree):-1:1
    if tree(i).leaf == 0 
         tree(i).p = 1-((1-tree(tree(i).NW).p) * (1-tree(tree(i).NE).p) * ...
            (1-tree(tree(i).SE).p) * (1-tree(tree(i).SW).p));
    end
end




maxDepth = oldMaxDepth;