%% initialization
clear; close all; clc;

% Uncomment to get repeatable runs
RandStream.setDefaultStream(RandStream('mt19937ar','seed',45));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Global data for constants or mission-wide data %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% PQ being built
global tree;

% maximum depth for the tree
global maxDepth;
maxDepth = 8+1; % recall root node starts at depth=1

% depth-dependent sensor performance
global alpha; % false positive
global beta;  % missed detection
% sensor perfomance
x = 1:10;
% alpha values parametrized by depth
alpha = 0.2 - 0.01*x;
% beta values parametrized by depth
beta = 0.3 - 0.02*x;

global lower_thresh;
global upper_thresh;
% Lower search decision thresholds for clearing a cell
lower_thresh = 0.0 + 0.005*x;
% Upper search decision thresholds for designating a cell as occupied
upper_thresh = 0.81 + 0.02*x;

% threshold to establish when two reals are the same
global epsilon;
epsilon = 0.00001;

%global pnull; % probability of the hypothesis target not present
%global belief_evol;

% constants to know where cell centers are; these should actually be
% initialized based on the data loaded from the prior.
global xslices;
global yslices;
edge = 256;
% Define constants
xslices = edge;
yslices = edge;

global cost_ascend; % cost increase for ascending
global bonus_descend; % bonus for descending
% for the time being disregard ascending/descending
cost_ascend = 0;
bonus_descend = 0;

global search_strategy;
% Search strategy codes
% 1: ICRA heuristic
% 2: Pure information gain
% 3: information gain with distance
search_strategy = 3;

global InformationGainFunction; % allows to swith between correct and heuristic IG
InformationGainFunction = @InformationGain;

global numRobots; % number of searchers
numRobots = 1;
global robotBounds; % bounds for each searcher
robotBounds = [0 0 256 256];
% numRobots = 2;
% robotBounds = [0 0 128 256;128 0 256 256];

% numRobots = 2;
% robotBounds = [0 0 1 256 256 5;0 0 6 256 256 9];

% numRobots = 4;
% robotBounds = [0 0 128 128;128 0 256 128;0 128 128 256;128 128 256 256];

% Time spent for a scanning step
global scanCost;
scanCost = 5;

% positions of the target(s)
global target;

global heightDist; % height difference between two levels in the tree
heightDist = 20;

% required confidence level for every cell in order to stop search with
% individual cell confidence test
global cell_confidence_requirement;
cell_confidence_requirement = 0.97;

% Costs for misclassificaions and correct classifications
global C00;
global C11;
global C10; % cost for false positive
global C01; % cost for missed detection
C11 = 0;
C00 = 0;
C10 = 10;
C01 = 1000;

global TicksAllowed; % search time left for each searcher
% overall search budget (aka maximum mission length)
maxTicks = 25000;

global StopCriterion;
% Stop criterion codes
% 1: stop when all cells have an entropy below a certain threshold, i.e.
% H(cell_confidence_requirement)
% 2: stop when the normalized tree entropy is below entropyFraction.
% Normalized tree entropy is the sum of entropy on the leaves divided by
% the number of leaves * H(globalEntropyLevel)
StopCriterion = 3;

% useful for criterion 3
global stop_epsilon;
stop_epsilon = 0.005;

% tree entropy level that should be not exceeded at the end (used only when
% StopCriterion = 1;
global globalEntropyLevel;
globalEntropyLevel = 0.97;

global entropyFraction;
entropyFraction = 0.01;

% if true cleared nodes won't be searched anymore
global PruneCleared;
PruneCleared = false;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% PROCESSING STARTS HERE                                                 %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%% run

% number of runs
RUNS = 100;

% initialize empty PQ
tree=CreateNode(0,0.5,1);
% load prior and prune it to 200 nodes
%BuildTreeMaxNodes(200,'init_belief_map_256.mat');
%BuildTreeMaxNodes(200,'256x256_uniform_boosted.mat');
BuildTreeMaxDepth(6,'256x256_uniform_boosted.mat');
% save tree so that it can be used for every iteration
savedTree = tree;

%% run

% Data structures to save results for post-mission analysis

% stores 1/0 if one or more targets were found
success = zeros(1,RUNS);
% stores the number of steps needed to complete the i-th run
time_steps = zeros(1,RUNS);
% stores the target positions found by the search algorithm
target_locations = cell(1,RUNS);
% number of targets found
num_targets_found = zeros(1,RUNS);
% number of false positives
num_false_positives = zeros(1,RUNS);
% number of missed targets
num_missed_targets = zeros(1,RUNS);
% cost of the mission
mission_cost = zeros(1,RUNS);

num_scans = zeros(numRobots,RUNS);
goal_information = cell(1,RUNS);
number_of_scans = cell(1,RUNS);


% load precomputed target positions; see generateTargetPositions.m
VariableTargets = load('TargetPositions.mat');
VariableTargets = VariableTargets.targets;

if RUNS > numel(VariableTargets)
    display('Given value for RUNS is too big; limiting to the size of VariableTargets');
    RUNS = numel(VariableTargets);
end

for i=81:100
    fprintf('Run #%d\n',i);
    tree = savedTree;
    TicksAllowed  = ones(1,numRobots)*maxTicks;
    target = VariableTargets{i};
    %[success(i),tloc,num_scans(:,i),goal_information{i}] = RunMission(true);
    path = sprintf('100RunsUniformDepth6Type2StoppingCriterion3Factor0005/RUN%d',i);
    [success(i),target_locations{i},number_of_scans{i},goal_information{i}] = RunMissionUniform(false,false,path);
    num_targets_found(i) = size(target_locations{i},2);
    time_steps(i) = maxTicks - TicksAllowed(1);
   
    [mission_cost(i) num_missed_targets(i) num_false_positives(i)] = ComputeDecisionCostsUniform(target_locations{i},target);
    fprintf('%d: Ticks taken %f\n',i,time_steps(i));
    fprintf('%d: Incurred Cost %f\n',i,mission_cost(i));
    save '100RunsUniformDepth5Type2StoppingCriterion3Factor01'
end
