function f = SplitNode(pos,offsets)

% splits a node into 4 children
% pos: index of the node to be split.
% returns the node just split

global tree;

f = tree(pos);

if f.leaf == 0 % error check
    error('Can''t split an internal node');
else
    f.leaf = 0; % leaf becomes internal
end

prob = 1 - sqrt(sqrt(1-f.p)); % uniformly spread probability of being occupied

if nargin == 1 % initialization without preallocation
    tree = [tree CreateNode(pos,prob,1)];
    f.NW = size(tree,2);
    tree(f.NW).positiveScans = f.positiveScans;
    tree(f.NW).negativeScans = f.negativeScans;
    
    tree = [tree CreateNode(pos,prob,2)];
    f.NE = size(tree,2);
    tree(f.NE).positiveScans = f.positiveScans;
    tree(f.NE).negativeScans = f.negativeScans;
    
    tree = [tree CreateNode(pos,prob,3)];
    f.SE = size(tree,2);
    tree(f.SE).positiveScans = f.positiveScans;
    tree(f.SE).negativeScans = f.negativeScans;
    
    tree = [tree CreateNode(pos,prob,4)];
    f.SW = size(tree,2);
    tree(f.SW).positiveScans = f.positiveScans;
    tree(f.SW).negativeScans = f.negativeScans;
    
    tree(pos) = f; % updates the tree
    
else % fast initialization with preallocation
    first_child_pos = offsets(tree(pos).depth+1) + ...
        ( pos - offsets(tree(pos).depth))*4;
    tree(first_child_pos) = CreateNode(pos,prob,1);
    tree(first_child_pos).positiveScans = f.positiveScans;
    tree(first_child_pos).negativeScans = f.negativeScans;
    tree(first_child_pos+1) = CreateNode(pos,prob,2);
    tree(first_child_pos+1).positiveScans = f.positiveScans;
    tree(first_child_pos+1).negativeScans = f.negativeScans;
    tree(first_child_pos+2) = CreateNode(pos,prob,3);
    tree(first_child_pos+2).positiveScans = f.positiveScans;
    tree(first_child_pos+2).negativeScans = f.negativeScans;
    tree(first_child_pos+3) = CreateNode(pos,prob,4);
    tree(first_child_pos+3).positiveScans = f.positiveScans;
    tree(first_child_pos+3).negativeScans = f.negativeScans;
    
    f.NW = first_child_pos;
    f.NE = first_child_pos+1;
    f.SE = first_child_pos+2;
    f.SW = first_child_pos+3;
    tree(pos) = f;
    
    
end
