function UpdatePosterior(z,pos)

% Updates all probabilities in the tree based on the last sensor reading
% and the prior
% z: sensor reading (1 detection, 0 no detection)
% pos: index of the square in the tree where sensing took place
% pos is s(t) in the WAFR paper
% z is Y_s(t) in the WAFR paper

global tree;
global pnull;
global epsilon;
global lower_thresh;


% start updating the quadrant where sensing happened

% this is equation 1 in the paper with c == s(t), hence
% the numerator is psi
%oldp = tree(pos).p;

tree(pos).lastScan = z;

newp = psi(z,pos)*tree(pos).p / ...
    ( phi(z,pos) * (1-tree(pos).p ) + psi(z,pos) * tree(pos).p ) ;

if ( newp > lower_thresh( tree(pos).depth ) )
    tree(pos).cleared = 0;
end

if newp > 1 + epsilon
    % tree(pos).p = 1;
    error('Found a basic update wrong...');
    
end

indexes = zeros(1,numel(tree)); % to keep track of which leafs were updated

if tree(pos).leaf == 0 % sensed and updated an internal node?
    % propagate update to descendants
    %    indexes = BayesUpdateTopDown(pos,newp,indexes);
  %  error('Updating an internal node...');
    BayesUpdateTopDown(pos,newp);
else
    tree(pos).p = newp;
    SetDecision(pos,z); % decisions are updated only for leaves;
    %   indexes(pos) = 1; % mark this leaf was already updated;
end


% propagate update to ancestors (always needed, unless it's the root)
while pos ~= 1
    pos = tree(pos).parent;
    tree(pos).p = 1-((1-tree(tree(pos).NW).p) * (1-tree(tree(pos).NE).p) * ...
        (1-tree(tree(pos).SE).p) * (1-tree(tree(pos).SW).p));
    if ( tree(pos).p > lower_thresh( tree(pos).depth ) )
        tree(pos).cleared = 0;
    end
end


% finally recompute probabilities for all internal nodes
% for i = numel(tree):-1:1
%     if tree(i).leaf == 0 % for every internal node
%         % consider  children contributions ; enforce constraint
%         tree(i).p = 1-((1-tree(tree(i).NW).p) * (1-tree(tree(i).NE).p) * ...
%             (1-tree(tree(i).SE).p) * (1-tree(tree(i).SW).p));
%         if ( tree(i).p > lower_thresh( tree(i).depth ) )
%             tree(i).cleared = 0;
%         end
%
%         if tree(i).p > 1 + epsilon
%             error('Internal node probability greater than 1');
%         end
%     end
% end

% now compute the probability of the null hypothesis (target not present)

pnull = 1-tree(1).p;

end

function f = phi(z,st)

global alpha;
global tree;

f = (1-z)*(1-alpha(tree(st).depth)) + z*alpha(tree(st).depth);
end

function f = psi(z,st)

global beta;
global tree;

f = (1-z)*beta(tree(st).depth) + z*(1-beta(tree(st).depth));
end

function f = theta(z,st,c)
if st == c
    f = psi(z,st);
else
    f = phi(z,st);
end
end