%
%  Plotting the initial prior belief distribution
%     - Used in 2009-ProbSearch paper
%
clc; close all; clear;

%------------------------
%  Probability map parameters
%   - B_0   : initial aggregate belief
%   - GRID_L: number of row/cols in finest resolution grid
B_0 = 0.75;
SEARCH_GRID_L = 256;


%------------------------
%  Create bimodal distribution
%

% % load('init_belief_map.mat');

% Allocate init_belief memory space
init_belief_map = zeros(SEARCH_GRID_L, SEARCH_GRID_L);


% Define initial belief distribution 

% first gaussian
mu1 = 0.10 * SEARCH_GRID_L;   sigma1 = 5.0 * SEARCH_GRID_L;
mu2 = 0.30 * SEARCH_GRID_L;   sigma2 = 7.0 * SEARCH_GRID_L;

for i = 1:SEARCH_GRID_L
  for j = 1:SEARCH_GRID_L
    init_belief_map(i,j) = 2*exp(-(i-mu1)^2/(2*sigma1) - (j-mu2)^2/(2*sigma2) );
  end     % end for(j)
end         % end for(i)

% second gaussian
mu1 = 0.60 * SEARCH_GRID_L;   sigma1 = 20.0 * SEARCH_GRID_L;
mu2 = 0.90 * SEARCH_GRID_L;   sigma2 = 35.0 * SEARCH_GRID_L;

for i = 1:SEARCH_GRID_L
  for j = 1:SEARCH_GRID_L
    init_belief_map(i,j) = init_belief_map(i,j) + exp(-(i-mu1)^2/(2*sigma1) - (j-mu2)^2/(2*sigma2) );
  end     % end for(j)
end         % end for(i)

% normalization
init_belief_map = init_belief_map./ (sum(sum(init_belief_map))/B_0);




% % Simple checks
% if ( (sum(Belief_0) ) ~= B_0)
%     1-(sum(Belief_0))
%     error('initial cell beliefs of search region does not sum to B(0)');
% end

%%% TEST CODE
% bar3(init_belief_map)





%------------------------
%  Plot bimodal distribution
%    - sub-sampled in order to help plot
%

init_belief_sm = init_belief_map(1:16:SEARCH_GRID_L, :);
init_belief_sm = init_belief_sm(:,1:16:SEARCH_GRID_L);

NUM_BARS = SEARCH_GRID_L/16;


min_Z = 0.0;


figure

set(gcf, 'Color', 'white');
set(gcf, 'Position', [680 406 880 694]);
hold off
h_reward = bar3( init_belief_sm, 1 );
axis square
% xlabel('Column j');  
% ylabel('Row i'); 
set(gca, 'FontSize', 12)
h_zlabel= zlabel('Initial Belief Distribution   '); set(gca, 'ZLim', [ min_Z, 1.5*max(max(init_belief_sm))], 'ZTick', 0:0.005:max(max(init_belief_sm)) , 'ZTickLabel', {0:0.005:max(max(init_belief_sm))}  );
set(h_zlabel,'FontSize', 16);
% set(gca, 'XTickLabel', {'10' '20' '30' '40' '50' '60' '70' '80' '90' '100'}); %set(gca,'YTick', [0 0.2 0.4 0.6 0.8 1]);
% set(gca, 'YLim', [0 11]); set(gca, 'YTickLabel', {'1' '2' '3' '4' '5' '6' '7' '8' '9' '10'}); %set(gca, 'XTick', [0 0.2 0.4 0.6 0.8 1]);
set(gca, 'XLim', [0.5, NUM_BARS+0.5], 'XTick', [], 'XTickLabel', {}, 'YLim', [0.5, NUM_BARS+0.5], 'YTick', [], 'YTickLabel', {});

axis ij
view([-45 30]); 

% title(sprintf('Single Cell, Min # looks:   B_{hi}=%1.2f, p_c^0=%1.2f', B_hi, p_c_0));
grid off




for i = 1:NUM_BARS                %(=length(h))
  % Initialize Zdata
  zdata = ones( size(get(h_reward(1), 'CData')) );
  
  j = 1;
  for k = 0:6:(6*NUM_BARS-6)
    zdata(k+1:k+6,:) = init_belief_sm(j,i);
    j = j+1;
  end
  set(h_reward(i),'Cdata',zdata)
end
colormap(bone(102800)); brighten(0.75)
%       cmap = colormap;
%       cmap(1,:) = [1,0.2,0.2];
%       colormap(cmap);

% - http://www.mathworks.com/access/helpdesk/help/techdoc/index.html?/access/helpdesk/help/techdoc/ref/caxis.html&http://www.mathworks.com/access/helpdesk/help/techdoc/ref/f16-6011.html

shading interp
for i = 1:NUM_BARS
  zdata = get(h_reward(i),'Zdata');
  set(h_reward(i),'Cdata',zdata)
  set(h_reward,'EdgeColor','k')
end
%%% http://www.mathworks.com/access/helpdesk/help/techdoc/index.html?/access/helpdesk/help/techdoc/creating_plots/f10-19972.html#f10-22737


% 
% % Cell labels
% h_label = zeros( size(init_belief_sm,1), size(init_belief_sm,2) );
% label_ctr = 0;
% for alpha_ind = 1:size(init_belief_sm,1)
%   for beta_ind = 1:size(init_belief_sm,2)
%    label_ctr = label_ctr + 1;
%     
%    h_label(label_ctr) = text( beta_ind-0.1, alpha_ind-0.1, min_Z, num2str( sub2ind([SEARCH_GRID_L, SEARCH_GRID_L], alpha_ind, beta_ind)  ) );
% %     if (expTDD(alpha_ind, beta_ind) == 0 )
% %       set(h_label(label_ctr), 'Visible', 'off');
% %     end
%   end
% end
% set(h_label, 'Color', 'black', 'FontWeight', 'bold')



% % Grid lines
% ctr = 0;
% for j = 0:NUM_BARS
%   for i = 0:NUM_BARS
%     ctr = ctr + 1;
%     h_line_row(ctr) = line([0.5,10.5],[i+0.5,i+0.5],[min_Z,min_Z]);
%     h_line_col(ctr) = line([j+0.5,j+0.5],[0.5,10.5],[min_Z,min_Z]);
% 
%   end
% end
% 
% set(h_line_row, 'Color', 'k', 'LineWidth', 2);
% set(h_line_col, 'Color', 'k', 'LineWidth', 2);
