clear; close all; clc;

% Uncomment to get repeatable runs
RandStream.setDefaultStream(RandStream('mt19937ar','seed',45));

global tree;
global alpha;
global beta;
global lower_thresh;
global upper_thresh;
global epsilon;
global pnull; % probability of the hypothesis target not present

global maxDepth;

global xslices;
global yslices;

global cost_ascend; % cost increase for ascending
global bonus_descend; % bonus for descending
global search_strategy;
global InformationGainFunction;

global TicksAllowed;
global robotBounds;
global numRobots;
global scanCost;

global belief_evol;
global target;

global heightDist;
global cell_confidence_requirement;

global C00;
global C11;
global C10;
global C01;

C11 = 0;
C00 = 0;
C10 = 10;
C01 = 1000;

%global GoalHistory;

%GoalHistory = [];

cell_confidence_requirement = 0.95;
% Search strategy codes
% 1: ICRA heuristic
% 2: Pure information gain
% 3: information gain with distance
search_strategy = 3;
InformationGainFunction = @InformationGain;

heightDist = 20;
edge = 256;

% for the time being disregard ascending/descending
cost_ascend = 0; %1000;
bonus_descend = 0;

% Define constants
xslices = edge;
yslices = edge;

% threshold to establish when two reals are the same
epsilon = 0.00001;

maxDepth = 8+1;     % recall root node starts at depth=1

% sensor perfomance
x = 1:10;
% these are the alpha values parametrized by depth
alpha = 0.2 - 0.01*x;
% these are the beta values parametrized by depth
beta = 0.3 - 0.02*x;

% Lower search decision thresholds for clearing a cell
lower_thresh = 0.0 + 0.01*x;

% Upper search decision thresholds for designating a cell as occupied
upper_thresh = 0.81 + 0.02*x;

% Define belief evolution vector
%  - initialize aggregate belief

% remember target must be at deepest resolution
%target = [ 50.5 150.5; 100.5 150.5;200.5 50.5]';     
target =...
[     27.5000   66.5000  243.5000;...
  150.5000  100.5000  186.5000];
% threshold to establish when two reals are the same
epsilon = 0.00001;


tree=CreateNode(0,0.5,1);
%BuildTreeMaxDepth(5,'256x256_uniform_boosted.mat');
BuildTreeMaxNodes(200,'256x256_uniform_boosted.mat');


numRobots = 1;
robotBounds = [0 0 256 256];

% numRobots = 2;
% robotBounds = [0 0 128 256;128 0 256 256];
 
% numRobots = 2;
% robotBounds = [0 0 3 256 256 4;0 0 5 256 256 9];

% numRobots = 4;
% robotBounds = [0 0 128 128;128 0 256 128;0 128 128 256;128 128 256 256];

scanCost = 5;

maxTicks = 15000;
TicksAllowed  = ones(1,numRobots)*maxTicks;
tic
[s targets] = RunMission(false);
toc
s
targets

decisions = [tree(1:numel(tree)).decision];
called = tree(decisions>0);
x = [called(1:numel(called)).xc];
y = [called(1:numel(called)).yc];

newTargets = [x ; y];
newTargets

ComputeDecisionCosts(targets,target)
ComputeDecisionCosts(newTargets,target)

fprintf('Ticks taken %f\n',maxTicks - TicksAllowed(1));