clear; close all; clc;

% Uncomment to get repeatable runs
RandStream.setDefaultStream(RandStream('mt19937ar','seed',45));

global tree;
global alpha;
global beta;
global lower_thresh;
global upper_thresh;
global treeFigureHandler;
global epsilon;
global pnull; % probability of the hypothesis target not present

global maxDepth;

global xslices;
global yslices;

global cost_ascend; % cost increase for ascending
global bonus_descend; % bonus for descending

edge = 256;

% for the time being disregard ascending/descending
cost_ascend = 1000;
bonus_descend = 0;

% Define constants
xslices = edge;
yslices = edge;

% threshold to establish when two reals are the same
epsilon = 0.00001;

maxDepth = 8+1;     % recall root node starts at depth=1

% sensor perfomance
x = 1:10;
% these are the alpha values parametrized by depth
alpha = 0.2 - 0.01*x;
% these are the beta values parametrized by depth
beta = 0.3 - 0.02*x;

% Lower search decision thresholds for clearing a cell
lower_thresh = 0.0 + 0.01*x;

% Upper search decision thresholds for designating a cell as occupied
upper_thresh = 0.81 + 0.01*x;

% Define belief evolution vector
%  - initialize aggregate belief
initProb = 0.5;
belief_evol = [initProb];

% target position  (x,y) positions in columns
% in a type one PQ there is always only one intruder
% target = [ 200  200]';
target = [ 150 150]';      % remember target must be at deepest resolution

% threshold to establish when two reals are the same
epsilon = 0.00001;

% create the starting empty tree
root = CreateNode(0,initProb,0);
root.xmin = 0;
root.ymin = 0;
root.xmax = edge;
root.ymax = edge;
root.xc = (root.xmax + root.xmin)/2;
root.yc = (root.ymax + root.ymin)/2;
tree = [root];

% setup figures
%gcmesh = figure;
%PlotEstimate(gcmesh);
% treeFigureHandler = DrawQuadTree;
% plot(target(1,:),target(2,:),'ro');

%BuildTreeMaxDepth(5,'init_belief_map_256.mat');
BuildTreeMaxNodes(200,'init_belief_map_256.mat');
% % % gcmesh = figure;
% % % PlotEstimate(gcmesh);
% % % treeFigureHandler = DrawQuadTree;

% saveTree = tree;

CheckTreeConsistency;

% BuildTreeMaxDepth(5,'init_belief_map_256.mat');
% % % % gcmesh = figure;
% % % % PlotEstimate(gcmesh);
% % % % treeFigureHandler = DrawQuadTree;
%
% CheckTreeConsistency; % set a first breakpoint here
%
% At this point you'll see 4 figures
% #1 is the plot of the prior for the tree built out of a fixed number
% of nodes
% #2 is the corresponding (irregular) quadtree subdivision
% #3 is the plot of the prior for a full tree with a given depth
% #4 is the corresponding (regular) quadtree subdivision
% Figure #1 seems wrong at first and is indeed counterintuitive. But after
% a while I convinced myself that the the peak at (240,144) is indeed
% higher than the one at (56,24) because it is the integral of a larger
% area (compare the corresponding tiles in the quadtree). In fact, if one
% plays a bit with the data cursor on the 3D figures one will see the
% probabilities indeed add up correctly. This is confirmed by the following
% three calls that refine the tree around the peak

% tree = saveTree;

% LocateAndSplit(200,136);
% LocateAndSplit(232,136);
% LocateAndSplit(232,168);


% Visualization flags
FLAG_SHOW_MAP = true;
FLAG_SHOW_3D = true;

if(FLAG_SHOW_MAP)
    gcmesh = figure;
    PlotEstimate(gcmesh);
    Movie1(1) = getframe(gcf);
end

treeFigureHandler = DrawQuadTree;
if(FLAG_SHOW_3D)
    [threeDHandler h] = Initialize3DPlot();
end

figure(treeFigureHandler); hold on;
Movie2(1) = getframe(gcf);

h_sensor(1) = plot3(-100, -100, max(get(gca, 'ZLim')), 'ro', 'MarkerSize', 10, 'MarkerFaceColor', 'r' );  % searcher position
h_sensor(2) = plot3(-100, -100, max(get(gca, 'ZLim')), 'rx', 'MarkerSize', 10 );                          % searcher next position
hold off;

cleared_cells = [];

CheckTreeConsistency; % set a second breakpoint here

pnull = 1-tree(1).p; % probability the target is outside

c_curr = 1; % position in the tree where the sensor is currently located;
% we assume in the beginning it is at the center, maximum height, i.e. root
% of the tree

c_next = 1;
c_goal = 2;

curPath = [];
% Define belief trajectory vector
%  - initialize with start location
belief_traj = c_curr;

sensing = false;

%=============================================
% SIMULATION
%  - SENSE-->PLAN-->ACT flow
%=============================================
for i = 1:1000
    % Time stamp
    fprintf('--> Time step %4d\n', i);
    
    
    %=========================
    % Searcher Information Gathering
    %  - observe desired location (if appropriate)
    %=========================
    
    %if there is no currently planned path need to make one
    if isempty(curPath)     
        %compute a new goal
        c_goal = GetMaximumPosition(c_curr);
        curPath = ComputeShortestPath(c_curr,c_goal)
    end
    % Determine if the sensor is on, dictated by whether at goal location
    %  - we don't (yet) sense while in transit
    if ( curPath(1) == c_goal )       % arrived at goal
        
        % read the sensor, and if necessary expands the tree
        [z,pos] = GetSensorReading(tree(c_goal).xc, tree(c_goal).yc, target);
        c_curr = pos;
        sensing = true;
        UpdatePosterior(z,pos);
        
        % Given current observation and updated map, define decision
        [isSearchDone, cleared_cells] = EvaluateSearchDecision( z, c_curr, cleared_cells );
        if (isSearchDone)
            disp('Search has been completed');
            break;
        end;
        curPath = [];
    else
        sensing = false;
        % Update to next position en route to goal
        curPath(1) = [];
        c_curr = curPath(1);
    end          % end if(check for transit)
    
    
    fprintf('  main.m: c_curr=%d, c_goal=%d\n', c_curr, c_goal);
    
    
    
    
    %=========================
    % Data structure maintenance
    %  - makes sure constraints are satisfied
    %=========================
    CheckTreeConsistency % Debug only;
    
    
    
    %=========================
    % Data logging
    %=========================
    % Augment belief evolution vector
    belief_evol = [belief_evol; tree(1).p];
    
    % Record searcher trajectory
    belief_traj = [belief_traj; c_curr];
    
    
    
    
    
    %=========================
    % Visualization handling
    %=========================
    if ( FLAG_SHOW_MAP )
       % if mod(i,10) == 0 % plot once in a while to save time...
            PlotEstimate(gcmesh);
             
      %  end
        
        % Annotating desired search location on probability map
        %  - need to recreate object since probability map figure is redrawn
        figure(gcmesh); hold on;
        h_goal = plot3(tree(c_goal).xc, tree(c_goal).yc, max(get(gca, 'ZLim')), 'rx', 'MarkerSize', 10 );
        hold off;
        Movie1(i+1) = getframe(gcf);
    end         % end if(check FLAG_SHOW_MAP flag)
    
    
    % Annotating desired search location on quad tree diagram
    %  - don't need to recreate object since quad tree figure is NOT redrawn
    set( h_sensor(1), 'XData', tree(c_curr).xc, 'YData', tree(c_curr).yc);
    set( h_sensor(2), 'XData', tree(c_goal).xc, 'YData', tree(c_goal).yc);
    
    
    if(FLAG_SHOW_3D)
        UpdatePos3D(c_curr,threeDHandler,h,sensing);
    end
    
    
    % Annotate any cleared cells on quadtree diagram
    %  - leave all annotations on figure
    if ( ~isempty(cleared_cells) )
        figure(treeFigureHandler); hold on;
        plot3( tree(cleared_cells(end)).xc, tree(cleared_cells(end)).yc, max(get(gca, 'ZLim')), 'ko', 'MarkerSize', 6 );
        hold off;
       
    end
    
     Movie2(i+1) = getframe(gcf);
    
end

movie2avi(Movie1,'posterior.avi','compression','none');
movie2avi(Movie2,'quadtree.avi','compression','none');
