function [maxNodeInd] = GetMaximumPositionUniform(c_curr,bounds)

% generates valid sense position with maximal objective value


% This version considers also the travel cost between the current position
% and the optimal next sense position. The parameter c_curr is the index in
% the tree where the robot performed the last sensing position


global tree;
global search_strategy;
global maxDepth;
if nargin < 2
    bounds = [tree(1).xmin tree(1).ymin 1 tree(1).xmax tree(1).ymax maxDepth];
end

%added so code that uses only x/y bounds doesnt break
if(size(bounds,2) < 6)
    bounds = [bounds(1) bounds(2) 1 bounds(3) bounds(4) maxDepth];
end

% Compute node with maximal objective value
maxNodeInd = 1;        % initialize as root
maxNodeVal = -inf;%tree(maxNodeInd).p * 4^(tree(maxNodeInd).depth-1) / 1;

l = [tree(1:numel(tree)).leaf];
leaves = tree(l>0);
offset = find(l>0);
offset = offset(1)-1;

if search_strategy > 1
    
    %find out which node yields the highest information gain and the largest
    %distance from the given node
    distances = zeros(length(leaves),1);
    informationGains = zeros(length(leaves),1);
    
    
    
    
    for which_ind = 1:size(leaves,2)
       % if tree(which_ind).leaf % considers leaves only (necessary for uniform)
         %   if ( inBounds(which_ind,bounds))     % check that this node is not cleared
                distances(which_ind) = StraightLineDistance(c_curr,which_ind+offset);
                informationGains(which_ind) = ExpectedInformationGainUniform(which_ind+offset);
          %  end
        %end
    end
    largestDistance = max(distances);
    largestExpectedInformationGain = max(informationGains);
    
    if ( largestExpectedInformationGain < 0 )
        largestExpectedInformationGain
        save('ErrorDump');
        error('Negative Information Gain?');
    end
    
end

% Search all leaves (linearly through tree data structure)
considered = 0;
for which_ind = 1:size(leaves,2)
    
    %if tree(which_ind).leaf
        if (inBounds(which_ind,bounds))     % check that this node is not cleared
            considered = considered+1;
            if ( which_ind+offset ~= c_curr ) % does not consider staying there (transit cost would be 0...)
                % Compute transit cost between current position and this element's position
                % min_cost = Distance(c_curr,which_ind);
                min_cost = StraightLineDistance(c_curr,which_ind+offset);
            else
                % Assume variable cost for staying in same location
                %  - low cost implies greater desire to 'exploit'
                %  - high cost implies greater desire to 'explore'
                %      (note: high cost might cause ringing to occur)
                % allows for exploitation computing an approximate value for
                % staying there (tryes a sibling rather than itself)
                parent = tree(c_curr).parent;
                if parent == 0 % special case for the root
                    parent = 2;   % will break if the tree has just one node;
                end
                sibling = c_curr+1;
                if sibling > tree(parent).SW % SW is the largest children of the common parent
                    sibling = tree(parent).NW;
                end
                min_cost = Distance(sibling,which_ind+offset);
                
            end
            
            % Evaluate objective function
            if search_strategy == 1  % ICRA heuristic
                objValue = tree(which_ind).p * 4^(tree(which_ind).depth-1) / min_cost;
            elseif search_strategy == 2 % pure information gain
                %trying with pure information gain
                objValue = informationGains(which_ind);
            elseif search_strategy == 3 %information gain and distance
                IGWeight = 0.5;
                if largestExpectedInformationGain > 0
                    objValue = IGWeight*(informationGains(which_ind)/largestExpectedInformationGain)...
                        + (1-IGWeight)*(1 - distances(which_ind)/largestDistance);
                else
                    objValue = (1 - distances(which_ind)/largestDistance);
                end
            else
                error('Uknown search strategy');
            end
            %trying with entropy
            %objValue = (-tree(which_ind).p*log(tree(which_ind).p)-(1-tree(which_ind).p)*log(1-tree(which_ind).p)) * 4^(tree(which_ind).depth-1) / min_cost;
            
            %
            % Compare with current maximal node
            if ( objValue > maxNodeVal )        % ignore equality case intentionally
                % Update to this node
                maxNodeVal = objValue;
                maxNodeInd = which_ind+offset;
            end         % end if(max value check)
            
            
        end         % end if(this node is not yet cleared)
   % end
    
end         % end for(which_node)

if maxNodeInd == 1
    bounds
    considered
    maxNodeVal
    largestExpectedInformationGain
    largestDistance
    for which_ind = 1:size(tree,2)
        
        if (inBounds(which_ind,bounds))     % check that this node is not cleared
            if largestExpectedInformationGain > 0 &&largestDistance
                objValue = IGWeight*(informationGains(which_ind)/largestExpectedInformationGain)...
                    + (1-IGWeight)*(1 - distances(which_ind)/largestDistance);
            else
                objValue = (1 - distances(which_ind)/largestDistance);
            end
        end
        
    end
end
end

function result = inBounds(pos,bounds)
global tree;
global PruneCleared;
result = tree(pos).xc > bounds(1) && tree(pos).yc > bounds(2) && ...
    tree(pos).depth >= bounds(3) && tree(pos).depth <= bounds(6) && ...
    tree(pos).xc < bounds(4) && tree(pos).yc < bounds(5);
if PruneCleared
    if tree(pos).cleared
        result = false;
    end
end
end

% % Return center coordinate of the node with maximal objective value
% x = tree(maxNodeInd).xc;
% y = tree(maxNodeInd).yc;

% Return instead the index of the node with maximal objective value

